@@ -188,10 +188,7 @@ class SiteTree(object):
 
         for item in sitetree:
             if sitetree_needs_caching:
-                if item in parents:
-                    item.has_children = True
-                else:
-                    item.has_children = False
+                item.has_children = False
 
                 if not hasattr(item, 'depth'):
                     item.depth = self.calculate_item_depth(alias, item.id)
@@ -398,7 +395,7 @@ class SiteTree(object):
                 parent_ids.append(branch_id)
             elif branch_id.isdigit():
                 parent_ids.append(branch_id)
-            elif branch_id.isalnum():
+            else:
                 parent_aliases.append(branch_id)
 
         menu_items = []
@@ -414,7 +411,9 @@ class SiteTree(object):
 
         # Parse titles for variables.
         menu_items = self.parse_titles(menu_items, context)
-        return self.apply_hook(menu_items, 'menu')
+        menu_items = self.apply_hook(menu_items, 'menu')
+        menu_items = self.update_has_children(tree_alias, menu_items, 'menu')
+        return menu_items
 
     def apply_hook(self, items, sender):
         """Applies item processing hook, registered with ``register_item_hook()``
@@ -454,7 +453,9 @@ class SiteTree(object):
         if current_item is not None:
             self.breadcrumbs_climber(tree_alias, current_item)
             self.cache_breadcrumbs.reverse()
-        return self.apply_hook(self.cache_breadcrumbs, 'breadcrumbs')
+        items = self.apply_hook(self.cache_breadcrumbs, 'breadcrumbs')
+        items = self.update_has_children(tree_alias, items, 'breadcrumbs')
+        return items
 
     def tree(self, tree_alias, context):
         """Builds and returns tree structure for 'sitetree_tree' tag."""
@@ -463,7 +464,9 @@ class SiteTree(object):
         if not sitetree_items:
             return ''
         tree_items = self.filter_items(self.get_children(tree_alias, None), 'sitetree')
-        return self.apply_hook(tree_items, 'sitetree')
+        tree_items = self.apply_hook(tree_items, 'sitetree')
+        tree_items = self.update_has_children(tree_alias, tree_items, 'sitetree')
+        return tree_items
 
     def children(self, parent_item, navigation_type, use_template, context):
         """Builds and returns site tree item children structure
@@ -479,21 +482,34 @@ class SiteTree(object):
         self.get_sitetree(tree_alias)
         # Mark path to current item.
         self.tree_climber(tree_alias, self.get_tree_current_item(tree_alias))
+
         tree_items = self.get_children(tree_alias, parent_item)
         tree_items = self.filter_items(tree_items, navigation_type)
         tree_items = self.apply_hook(tree_items, '%s.children' % navigation_type)
+        tree_items = self.update_has_children(tree_alias, tree_items, navigation_type)
+
         my_template = template.loader.get_template(use_template)
-        my_context = template.Context({'sitetree_items': tree_items, 'user': context['user']})
-        return my_template.render(my_context)
+        context.update({'sitetree_items': tree_items})
+        return my_template.render(context)
 
     def get_children(self, tree_alias, item):
         tree_alias = self.resolve_tree_i18_alias(tree_alias)
         return self.get_cache_entry('parents', tree_alias)[item]
 
+    def update_has_children(self, tree_alias, tree_items, navigation_type):
+        """Updates 'has_children' attribute for tree items."""
+        items = []
+        for tree_item in tree_items:
+            children = self.get_children(tree_alias, tree_item)
+            children = self.filter_items(children, navigation_type)
+            children = self.apply_hook(children, '%s.has_children' % navigation_type)
+            tree_item.has_children = len(children)>0
+            items.append(tree_item)
+        return items
+
     def filter_items(self, items, navigation_type=None):
         """Filters site tree item's children if hidden and by navigation type.
         NB: We do not apply any filters to sitetree in admin app.
-
         """
         items_out = copy(items)
         if self.global_context.current_app != 'admin':
@@ -540,13 +556,16 @@ class SiteTree(object):
         if context is None:
             context = self.global_context
 
-        # Variable names are not empty do not allow commas and spaces.
-        varname = varname.strip()
-        if varname != ''  and varname.find(' ') == -1 and ',' not in varname:
+        if isinstance(varname, template.FilterExpression):
+            varname = varname.resolve(context)
+        else:
+            varname = varname.strip()
+
             try:
                 varname = template.Variable(varname).resolve(context)
             except template.VariableDoesNotExist:
                 varname = varname
+
         return varname
 
     def parse_titles(self, items, context=None):
